@@ -33,7 +33,12 @@ float GRquantamatrix[DV_COUNT][2 * DV_COUNT];
 float GRnutslots[DV_COUNT];
 int GRfoodslots[2 * DV_COUNT];
 int GRsaveslots[2 * DV_COUNT];
+int GRfoodlevel[2 * DV_COUNT];
+int GRlowestlevelchange[2 * DV_COUNT];
+int GRgroupsremaining[2 * DV_COUNT][DV_COUNT];
 int GRtotalquanta;
+int GRfoodcount;
+int GRnutcount;
 float GRbestvariance;
 
 void add_foods(int label)
@@ -104,10 +109,21 @@ if (strcmp(food_work.name,"") == 0) return;
 if (  label)
  {
  header("NUT:  Add a Labeled Food");
- printf("\n\n\n    Now select the ingredients in the order they appear on the ingredients");
+ printf("\n    Now select the ingredients in the order they appear on the ingredients");
  printf("\n    statement.  Don't worry if you can't find some of the ingredients, but");
  printf("\n    for all of them that you do find, keep the order the same.");
- spacer(5);
+ printf("\n");
+ printf("\n    Here are some additional commands that facilitate the creation of");
+ printf("\n    the ordered ingredients statement.  (The symbol \"#\" stands for a number");
+ printf("\n    representing an ingredient in the list):");
+ printf("\n");
+ printf("\n         #         Delete this ingredient");
+ printf("\n         #m#       Move this ingredient to the new position");
+ printf("\n         #(        Begin here a group that constitutes a single ingredient");
+ printf("\n         #)        End here a group that constitutes a single ingredient");
+ printf("\n         #!        Remove a group indicator from this ingredient");
+ printf("\n");
+ spacer(14);
  printf("\nPress <enter> to continue...");
  junk = get_int();
  }
@@ -119,7 +135,7 @@ for ( ; ; )
    {
    if (! label) header("NUT:  Add a Recipe");
    if (  label) header("NUT:  Add a Labeled Food");
-   if (! recipe_show(0))
+   if (! recipe_show(0,label?1:0))
     {
     printf("\nPress <enter> to continue...");
     junk = get_int();
@@ -133,7 +149,7 @@ for ( ; ; )
      if (! label)
       {
       header("NUT:  Add a Recipe");
-      recipe_show(0);
+      recipe_show(0,0);
       printf("\nDo you want to save recipe in the food database?  (y/n):  ");
       junk = get_char();
       if (junk == 'n' || junk == 'N')
@@ -148,7 +164,7 @@ for ( ; ; )
       {
       if (! label) header("NUT:  Add a Recipe");
       if (  label) header("NUT:  Add a Labeled Food");
-      recipe_show(0);
+      recipe_show(0,label?1:0);
       if (! label) printf("\nDo you want to add more foods to recipe?  (y/n):  ");
       if (  label) printf("\nDo you want to add more ingredients?  (y/n):  ");
       junk = get_char();
@@ -221,7 +237,7 @@ for ( ; ; )
         food_work.grams = savegrams;
         food_work.nutrient[WATER] = savewater;
         new_recipe_show();
-        printf("\nType new weight of serving {xxxg for grams, xxxo for oz}:  ");
+        printf("\nType new weight of serving {#g for grams, #o for oz}:  ");
         get_recipe_qty(&food_work.grams);
         food_work.nutrient[WATER] -= savegrams - food_work.grams;
         new_recipe_show();
@@ -260,7 +276,8 @@ for ( ; ; )
   token = strtok(buff, ", ");
   token = strtok(NULL,", ");
   if (strcmp(key,"") == 0 || modfood == 0) break;
-  modify_recipe_food(modfood,token);
+  if (! label) modify_recipe_food(modfood,token);
+  if (  label) modify_label_food(modfood,key);
   strcpy(key,"");
   }
  ratio = -383838;
@@ -299,6 +316,7 @@ for ( ; ; )
    } 
   memcpy(new_recipe,food_ptr,sizeof(struct food));
   new_recipe->grams *= ratio;
+  if (label) new_recipe->refuse = 0;
   order_new_recipe();
   }
  }
@@ -326,19 +344,55 @@ else if (strchr(buff,'o') != NULL) *result = (float) atof(buff) * 28.35;
 else *result = atof(buff);
 }
 
-int recipe_show(int trace)
+int recipe_show(int trace, int level)
 {
 struct food *recipe_ptr = &recipe_root;
-int count = 0;
+int i = 0, count = 0, cumlevel = 0, trimlength = 60;
+char nametrim[61];
 printf("Recipe Name:  %-s\n\n",food_work.name);
 while (recipe_ptr->next != NULL)
  {
  recipe_ptr = recipe_ptr->next;
  count++;
- if (options.grams && ! trace) printf("%2d. %-60s    %9.1f g\n",count,recipe_ptr->name,recipe_ptr->grams);
- if (options.grams && trace && recipe_ptr->grams >  0) printf("%2d. %-60s    %9.2f g\n",count,recipe_ptr->name,recipe_ptr->grams);
- if (options.grams && trace && recipe_ptr->grams == 0) printf("%2d. %-60s      %9s\n",count,recipe_ptr->name,"trace");
- if (!options.grams) printf("%2d. %-60s    %8.1f oz\n",count,recipe_ptr->name,recipe_ptr->grams/28.35);
+ if (! level)
+  {
+  if (options.grams) printf("%2d. %-60s    %9.1f g\n",count,recipe_ptr->name,recipe_ptr->grams);
+  if (!options.grams) printf("%2d. %-60s    %8.1f oz\n",count,recipe_ptr->name,recipe_ptr->grams/28.35);
+  }
+ if (level)
+  {
+  printf("%2d. ",count);
+  for (i = 0; i < cumlevel; i++) printf(" ");
+  if (recipe_ptr->refuse > 0) for (i = 0; i < recipe_ptr->refuse; i++) printf("(");
+  if (recipe_ptr->refuse > 0) cumlevel += recipe_ptr->refuse;
+  for (i = 0; i < 60; i++) nametrim[i] = ' ';
+  trimlength = 60 - cumlevel;
+  for (i = 0; i < trimlength; i++) 
+   {
+   if (recipe_ptr->name[i] == '\0') break;
+   nametrim[i] = recipe_ptr->name[i];
+   }
+  if (recipe_ptr->refuse < 0 && cumlevel + recipe_ptr->refuse < 0) recipe_ptr->refuse = 0 - cumlevel; 
+  if (recipe_ptr->refuse < 0) 
+   {
+   for (i = 0; i < trimlength; i++) 
+    {
+    if (recipe_ptr->name[i] == '\0') break;
+    nametrim[i] = recipe_ptr->name[i];
+    }
+   for (i = recipe_ptr->refuse; i < 0; i++) nametrim[trimlength+i] = ')';
+   cumlevel += recipe_ptr->refuse;
+   }
+/*
+  if (recipe_ptr->next == NULL && cumlevel > 0) for (i = cumlevel - recipe_ptr->refuse; i > 0; i--) nametrim[trimlength-i] = ')';
+*/
+  nametrim[trimlength] = '\0';
+  printf("%-s",nametrim);
+  if (options.grams && ! trace) printf("    %9.1f g\n",recipe_ptr->grams);
+  if (options.grams && trace && recipe_ptr->grams >  0) printf("    %9.2f g\n",recipe_ptr->grams);
+  if (options.grams && trace && recipe_ptr->grams == 0) printf("      %9s\n","trace");
+  if (!options.grams) printf("    %8.1f oz\n",recipe_ptr->grams/28.35);
+  }
  }
 if (count == 0) 
  {
@@ -385,7 +439,7 @@ for ( ; ; )
  }
 guess_recipe();
 for (i = 0; i < NUTRIENT_COUNT; i++) if (test_for_negative_zero(&food_work.nutrient[i])) food_work.nutrient[i] = -1;
-printf("\nDo you want to again edit the nutrient values (y/n)?  ");
+printf("\nDo you want to again view or edit the nutrient values (y/n)?  ");
 i = get_char();
 if (i != 'N' && i != 'n') screen_subvert_label(0);
 for (i = 0; i < NUTRIENT_COUNT; i++) if (food_work.nutrient[i] == -1) food_work.nutrient[i] = NoData;
@@ -404,30 +458,30 @@ for ( ; ; )
  yn = '\0';
  header("NUT:  Add a Labeled Food");
  printf("\n      Change group of nutrients to view?\n\n\n");
- for (i = 0; i < (dv ? dv + 1 : MaxScreen); i++) printf("                    %d  --  %s\n\n",i+1,ScreenTitle[i]);
- spacer((dv ? dv  + 1: MaxScreen) * 2 + 4);
- printf("\nEnter your choice (just <enter> to quit):  ");
- screen_choice = get_int();
- if (screen_choice < 1 || screen_choice > (dv ? dv + 1 : MaxScreen))
-  {
-  while (yn != 'Y' && yn != 'y' && yn != 'N' && yn != 'n')
-   {
-   header("NUT:  Add a Labeled Food");
-   spacer(-1);
-   printf("Do you want to further edit the nutrient values (y/n)?  ");
-   yn = get_char();
-   if (yn == 'N' || yn == 'n') 
-    {
-    options.screen = savescreen;
-    return;
-    }
-   if (yn == 'Y' || yn == 'y') break;
-   }
-  }
- if (yn == 'Y' || yn == 'y') continue;
- options.screen = screen_choice - 1;
- set_nut_table();
- }
+for (i = 0; i < (dv ? dv + 1 : MaxScreen); i++) printf("                    %d  --  %s\n\n",i+1,ScreenTitle[i]);
+spacer((dv ? dv  + 1: MaxScreen) * 2 + 4);
+printf("\nEnter your choice (just <enter> to quit):  ");
+screen_choice = get_int();
+if (screen_choice < 1 || screen_choice > (dv ? dv + 1 : MaxScreen))
+{
+while (yn != 'Y' && yn != 'y' && yn != 'N' && yn != 'n')
+{
+header("NUT:  Add a Labeled Food");
+spacer(-1);
+printf("Do you want to further edit the nutrient values (y/n)?  ");
+yn = get_char();
+if (yn == 'N' || yn == 'n') 
+{
+options.screen = savescreen;
+return;
+}
+if (yn == 'Y' || yn == 'y') break;
+}
+}
+if (yn == 'Y' || yn == 'y') continue;
+options.screen = screen_choice - 1;
+set_nut_table();
+}
 }
 
 void set_nut_table()
@@ -435,35 +489,35 @@ void set_nut_table()
 float tempfloat;
 int nutnum;
 for ( ; ; )
- {
- header("NUT:  Add a Labeled Food");
- label_nut_list(-1);
- printf("\nType number of nutrient to set (or <enter> to quit):  ");
- nutnum = get_int();
- if (nutnum < 1 || nutnum > *ScreenMap[options.screen]) return;
- nutnum = ScreenMap[options.screen][nutnum];
+{
+header("NUT:  Add a Labeled Food");
+label_nut_list(-1);
+printf("\nType number of nutrient to set (or <enter> to quit):  ");
+nutnum = get_int();
+if (nutnum < 1 || nutnum > *ScreenMap[options.screen]) return;
+nutnum = ScreenMap[options.screen][nutnum];
 
- header("NUT:  Add a Labeled Food");
- label_nut_list(nutnum);
- printf("\nEnter value for %s or <enter> to blank value:  ",Nutrient[nutnum]);
- get_nut_value(&tempfloat);
- if (options.screen == 0 && (nutnum != FAMS && nutnum != FAPU && nutnum != OMEGA6 && nutnum != LA && nutnum != AA && nutnum != OMEGA3 && nutnum != ALA && nutnum != EPA && nutnum != DHA && nutnum != CHO_NONFIB)) tempfloat = tempfloat / 100 * DVBase[nutnum];
- food_work.nutrient[nutnum] = tempfloat;
- if (nutnum == VITE) food_work.nutrient[TOCPHA] = tempfloat * 2/3;
- if (nutnum == LA  ) food_work.nutrient[F18D2] = tempfloat;
- if (nutnum == AA  ) food_work.nutrient[F20D4] = tempfloat;
- if (nutnum == ALA ) food_work.nutrient[F18D3] = tempfloat;
- if (nutnum == EPA ) food_work.nutrient[F20D5] = tempfloat;
- if (nutnum == DHA ) food_work.nutrient[F22D6] = tempfloat;
- if (nutnum == ENERC_KCAL ) food_work.nutrient[ENERC_KJ] = tempfloat * 4.1835;
- if (nutnum == TOCPHA) food_work.nutrient[VITE] = tempfloat * 3/2;
- if (nutnum == F18D2 ) food_work.nutrient[LA] = tempfloat;
- if (nutnum == F20D4 ) food_work.nutrient[AA] = tempfloat;
- if (nutnum == F18D3 ) food_work.nutrient[ALA] = tempfloat;
- if (nutnum == F20D5 ) food_work.nutrient[EPA] = tempfloat;
- if (nutnum == F22D6 ) food_work.nutrient[DHA] = tempfloat;
- if (nutnum == ENERC_KJ ) food_work.nutrient[ENERC_KCAL] = tempfloat / 4.1835;
- }
+header("NUT:  Add a Labeled Food");
+label_nut_list(nutnum);
+printf("\nEnter value for %s or <enter> to blank value:  ",Nutrient[nutnum]);
+get_nut_value(&tempfloat);
+if (options.screen == 0 && (nutnum != FAMS && nutnum != FAPU && nutnum != OMEGA6 && nutnum != LA && nutnum != AA && nutnum != OMEGA3 && nutnum != ALA && nutnum != EPA && nutnum != DHA && nutnum != CHO_NONFIB)) tempfloat = tempfloat / 100 * DVBase[nutnum];
+food_work.nutrient[nutnum] = tempfloat;
+if (nutnum == VITE) food_work.nutrient[TOCPHA] = tempfloat * 2/3;
+if (nutnum == LA  ) food_work.nutrient[F18D2] = tempfloat;
+if (nutnum == AA  ) food_work.nutrient[F20D4] = tempfloat;
+if (nutnum == ALA ) food_work.nutrient[F18D3] = tempfloat;
+if (nutnum == EPA ) food_work.nutrient[F20D5] = tempfloat;
+if (nutnum == DHA ) food_work.nutrient[F22D6] = tempfloat;
+if (nutnum == ENERC_KCAL ) food_work.nutrient[ENERC_KJ] = tempfloat * 4.1835;
+if (nutnum == TOCPHA) food_work.nutrient[VITE] = tempfloat * 3/2;
+if (nutnum == F18D2 ) food_work.nutrient[LA] = tempfloat;
+if (nutnum == F20D4 ) food_work.nutrient[AA] = tempfloat;
+if (nutnum == F18D3 ) food_work.nutrient[ALA] = tempfloat;
+if (nutnum == F20D5 ) food_work.nutrient[EPA] = tempfloat;
+if (nutnum == F22D6 ) food_work.nutrient[DHA] = tempfloat;
+if (nutnum == ENERC_KJ ) food_work.nutrient[ENERC_KCAL] = tempfloat / 4.1835;
+}
 }
 
 void label_nut_list(int selection)
@@ -471,25 +525,25 @@ void label_nut_list(int selection)
 int count;
 int intdiv = *ScreenMap[options.screen] / 2 + (*ScreenMap[options.screen] % 2 == 0 ? 0 : 1);
 for ( count = 0 ; count <= *ScreenMap[options.screen] / 2 - (*ScreenMap[options.screen] % 2 == 0 ? 1 : 0); count++)
- {
- if (options.screen == 0)
-  {
-  if (food_work.nutrient[ScreenMap[options.screen][count+1]] >= 0 && ScreenMap[options.screen][count+1] != FAMS && ScreenMap[options.screen][count+1] != FAPU && ScreenMap[options.screen][count+1] != OMEGA6 && ScreenMap[options.screen][count+1] != LA && ScreenMap[options.screen][count+1] != AA && ScreenMap[options.screen][count+1] != OMEGA3 && ScreenMap[options.screen][count+1] != ALA && ScreenMap[options.screen][count+1] != EPA && ScreenMap[options.screen][count+1] != DHA && ScreenMap[options.screen][count+1] != CHO_NONFIB) printf("     %2d%s %-14s%7.0f%% DV   ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],food_work.nutrient[ScreenMap[options.screen][count+1]]/DVBase[ScreenMap[options.screen][count+1]]*100);
-  if (food_work.nutrient[ScreenMap[options.screen][count+1]] <  0 && ScreenMap[options.screen][count+1] != FAMS && ScreenMap[options.screen][count+1] != FAPU && ScreenMap[options.screen][count+1] != OMEGA6 && ScreenMap[options.screen][count+1] != LA && ScreenMap[options.screen][count+1] != AA && ScreenMap[options.screen][count+1] != OMEGA3 && ScreenMap[options.screen][count+1] != ALA && ScreenMap[options.screen][count+1] != EPA && ScreenMap[options.screen][count+1] != DHA && ScreenMap[options.screen][count+1] != CHO_NONFIB) printf("     %2d%s %-14s%7s%% DV   ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],"       ");
-  if (food_work.nutrient[ScreenMap[options.screen][count+1]] >= 0 && (ScreenMap[options.screen][count+1] == FAMS || ScreenMap[options.screen][count+1] == FAPU || ScreenMap[options.screen][count+1] == OMEGA6 || ScreenMap[options.screen][count+1] == LA || ScreenMap[options.screen][count+1] == AA || ScreenMap[options.screen][count+1] == OMEGA3 || ScreenMap[options.screen][count+1] == ALA || ScreenMap[options.screen][count+1] == EPA || ScreenMap[options.screen][count+1] == DHA || ScreenMap[options.screen][count+1] == CHO_NONFIB)) printf("     %2d%s %-14s %7.2f %-3s  ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],food_work.nutrient[ScreenMap[options.screen][count+1]],Unit[ScreenMap[options.screen][count+1]]);
-  if (food_work.nutrient[ScreenMap[options.screen][count+1]] <  0 && (ScreenMap[options.screen][count+1] == FAMS || ScreenMap[options.screen][count+1] == FAPU || ScreenMap[options.screen][count+1] == OMEGA6 || ScreenMap[options.screen][count+1] == LA || ScreenMap[options.screen][count+1] == AA || ScreenMap[options.screen][count+1] == OMEGA3 || ScreenMap[options.screen][count+1] == ALA || ScreenMap[options.screen][count+1] == EPA || ScreenMap[options.screen][count+1] == DHA || ScreenMap[options.screen][count+1] == CHO_NONFIB)) printf("     %2d%s %-14s %7s %-3s  ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],"       ",Unit[ScreenMap[options.screen][count+1]]);
-  if (count+intdiv < *ScreenMap[options.screen] && food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]] >= 0) printf("    %2d%s %-14s%7.0f%% DV\n",count+1+intdiv,ScreenMap[options.screen][count+1+intdiv]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1+intdiv]],food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]]/DVBase[ScreenMap[options.screen][count+1+intdiv]]*100);
-  if (count+intdiv < *ScreenMap[options.screen] && food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]] <  0) printf("    %2d%s %-14s%7s%% DV\n",count+1+intdiv,ScreenMap[options.screen][count+1+intdiv]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1+intdiv]],"       ");
-  }
- else
-  {
-  if (food_work.nutrient[ScreenMap[options.screen][count+1]] >= 0) printf("     %2d%s %-14s %7.2f %-3s  ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],food_work.nutrient[ScreenMap[options.screen][count+1]],Unit[ScreenMap[options.screen][count+1]]);
-  if (food_work.nutrient[ScreenMap[options.screen][count+1]] <  0) printf("     %2d%s %-14s %7s %-3s  ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],"       ",Unit[ScreenMap[options.screen][count+1]]);
-  if (count+intdiv < *ScreenMap[options.screen] && food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]] >= 0) printf("    %2d%s %-14s %7.2f %-3s    \n",count+1+intdiv,ScreenMap[options.screen][count+1+intdiv]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1+intdiv]],food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]],Unit[ScreenMap[options.screen][count+1+intdiv]]);
-  if (count+intdiv < *ScreenMap[options.screen] && food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]] <  0) printf("    %2d%s %-14s %7s %-3s    \n",count+1+intdiv,ScreenMap[options.screen][count+1+intdiv]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1+intdiv]],"       ",Unit[ScreenMap[options.screen][count+1+intdiv]]);
-  }
- if (count+intdiv >= *ScreenMap[options.screen]) printf("\n");
- }
+{
+if (options.screen == 0)
+{
+if (food_work.nutrient[ScreenMap[options.screen][count+1]] >= 0 && ScreenMap[options.screen][count+1] != FAMS && ScreenMap[options.screen][count+1] != FAPU && ScreenMap[options.screen][count+1] != OMEGA6 && ScreenMap[options.screen][count+1] != LA && ScreenMap[options.screen][count+1] != AA && ScreenMap[options.screen][count+1] != OMEGA3 && ScreenMap[options.screen][count+1] != ALA && ScreenMap[options.screen][count+1] != EPA && ScreenMap[options.screen][count+1] != DHA && ScreenMap[options.screen][count+1] != CHO_NONFIB) printf("     %2d%s %-14s%7.0f%% DV   ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],food_work.nutrient[ScreenMap[options.screen][count+1]]/DVBase[ScreenMap[options.screen][count+1]]*100);
+if (food_work.nutrient[ScreenMap[options.screen][count+1]] <  0 && ScreenMap[options.screen][count+1] != FAMS && ScreenMap[options.screen][count+1] != FAPU && ScreenMap[options.screen][count+1] != OMEGA6 && ScreenMap[options.screen][count+1] != LA && ScreenMap[options.screen][count+1] != AA && ScreenMap[options.screen][count+1] != OMEGA3 && ScreenMap[options.screen][count+1] != ALA && ScreenMap[options.screen][count+1] != EPA && ScreenMap[options.screen][count+1] != DHA && ScreenMap[options.screen][count+1] != CHO_NONFIB) printf("     %2d%s %-14s%7s%% DV   ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],"       ");
+if (food_work.nutrient[ScreenMap[options.screen][count+1]] >= 0 && (ScreenMap[options.screen][count+1] == FAMS || ScreenMap[options.screen][count+1] == FAPU || ScreenMap[options.screen][count+1] == OMEGA6 || ScreenMap[options.screen][count+1] == LA || ScreenMap[options.screen][count+1] == AA || ScreenMap[options.screen][count+1] == OMEGA3 || ScreenMap[options.screen][count+1] == ALA || ScreenMap[options.screen][count+1] == EPA || ScreenMap[options.screen][count+1] == DHA || ScreenMap[options.screen][count+1] == CHO_NONFIB)) printf("     %2d%s %-14s %7.2f %-3s  ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],food_work.nutrient[ScreenMap[options.screen][count+1]],Unit[ScreenMap[options.screen][count+1]]);
+if (food_work.nutrient[ScreenMap[options.screen][count+1]] <  0 && (ScreenMap[options.screen][count+1] == FAMS || ScreenMap[options.screen][count+1] == FAPU || ScreenMap[options.screen][count+1] == OMEGA6 || ScreenMap[options.screen][count+1] == LA || ScreenMap[options.screen][count+1] == AA || ScreenMap[options.screen][count+1] == OMEGA3 || ScreenMap[options.screen][count+1] == ALA || ScreenMap[options.screen][count+1] == EPA || ScreenMap[options.screen][count+1] == DHA || ScreenMap[options.screen][count+1] == CHO_NONFIB)) printf("     %2d%s %-14s %7s %-3s  ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],"       ",Unit[ScreenMap[options.screen][count+1]]);
+if (count+intdiv < *ScreenMap[options.screen] && food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]] >= 0) printf("    %2d%s %-14s%7.0f%% DV\n",count+1+intdiv,ScreenMap[options.screen][count+1+intdiv]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1+intdiv]],food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]]/DVBase[ScreenMap[options.screen][count+1+intdiv]]*100);
+if (count+intdiv < *ScreenMap[options.screen] && food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]] <  0) printf("    %2d%s %-14s%7s%% DV\n",count+1+intdiv,ScreenMap[options.screen][count+1+intdiv]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1+intdiv]],"       ");
+}
+else
+{
+if (food_work.nutrient[ScreenMap[options.screen][count+1]] >= 0) printf("     %2d%s %-14s %7.2f %-3s  ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],food_work.nutrient[ScreenMap[options.screen][count+1]],Unit[ScreenMap[options.screen][count+1]]);
+if (food_work.nutrient[ScreenMap[options.screen][count+1]] <  0) printf("     %2d%s %-14s %7s %-3s  ",count+1,ScreenMap[options.screen][count+1]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1]],"       ",Unit[ScreenMap[options.screen][count+1]]);
+if (count+intdiv < *ScreenMap[options.screen] && food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]] >= 0) printf("    %2d%s %-14s %7.2f %-3s    \n",count+1+intdiv,ScreenMap[options.screen][count+1+intdiv]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1+intdiv]],food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]],Unit[ScreenMap[options.screen][count+1+intdiv]]);
+if (count+intdiv < *ScreenMap[options.screen] && food_work.nutrient[ScreenMap[options.screen][count+1+intdiv]] <  0) printf("    %2d%s %-14s %7s %-3s    \n",count+1+intdiv,ScreenMap[options.screen][count+1+intdiv]==selection ? "*" : ".",Nutrient[ScreenMap[options.screen][count+1+intdiv]],"       ",Unit[ScreenMap[options.screen][count+1+intdiv]]);
+}
+if (count+intdiv >= *ScreenMap[options.screen]) printf("\n");
+}
 spacer(count);
 }
 
@@ -503,38 +557,117 @@ else *dest = (atof(buff));
 
 void guess_recipe()
 {
-int i, j, savegramoption;
+int i, j, savegramoption, cumlevel = 0, ok = 0;
 int gt0dvnuts[DV_COUNT];
+int remaininggroupsassigner[DV_COUNT];
+int levelcumbuf[DV_COUNT] = {0,0,0,0,0,0,0,0,0,0,0,0};
+int levelminbuf[DV_COUNT] = {0,0,0,0,0,0,0,0,0,0,0,0};
+int parentheses[2 * DV_COUNT];
 float gt0dvnutfactors[DV_COUNT];
-int nutcount = 0;
 char *token;
 
 struct food *recipe_ptr;
 struct food *foodlist[2 * DV_COUNT];
-int foodcount = 0;
 
 GRtotalquanta = 100;
+GRfoodcount = 0;
+GRnutcount = 0;
+for (i = 0; i < DV_COUNT; i++) remaininggroupsassigner[i] = 0;
+
+for (j = 0; j < DV_COUNT; j++) if (food_work.nutrient[DVMap[j]] > 0) gt0dvnuts[GRnutcount++] = DVMap[j];
 
-for (j = 0; j < DV_COUNT; j++) if (food_work.nutrient[DVMap[j]] > 0) gt0dvnuts[nutcount++] = DVMap[j];
 recipe_ptr = &recipe_root;
-while (recipe_ptr->next != NULL && foodcount < 2 * DV_COUNT)
+while (recipe_ptr->next != NULL && GRfoodcount < 2 * DV_COUNT)
+{
+parentheses[GRfoodcount] = recipe_ptr->next->refuse;
+foodlist[GRfoodcount++] = recipe_ptr->next;
+recipe_ptr = recipe_ptr->next;
+}
+
+for (j = 0; j < GRnutcount; j++) gt0dvnutfactors[j] = food_work.grams / (100 * food_work.nutrient[gt0dvnuts[j]]);
+for (j = 0; j < GRnutcount; j++) for (i = 0; i < GRfoodcount; i++) GRquantamatrix[j][i] = foodlist[i]->nutrient[gt0dvnuts[j]] * gt0dvnutfactors[j];
+
+for (i = GRfoodcount - 1; i >= 0; i--) cumlevel += parentheses[i];
+if (cumlevel != 0) parentheses[GRfoodcount - 1] -= cumlevel;
+cumlevel = 1;
+
+while (1)
  {
- foodlist[foodcount++] = recipe_ptr->next;
- recipe_ptr = recipe_ptr->next;
+ for (i = GRfoodcount - 1; i >= 0; i--) if (parentheses[i] > -2 && parentheses[i] < 2) ok = 1;
+ if (ok) break;
+ for (i = GRfoodcount - 1; i >= 0; i--)
+  {
+  if (parentheses[i] < -1) parentheses[i]++;
+  if (parentheses[i] > -1) parentheses[i]--;
+  }
  }
-for (j = 0; j < nutcount; j++) gt0dvnutfactors[j] = food_work.grams / (100 * food_work.nutrient[gt0dvnuts[j]]);
-for (j = 0; j < nutcount; j++) for (i = 0; i < foodcount; i++) GRquantamatrix[j][i] = foodlist[i]->nutrient[gt0dvnuts[j]] * gt0dvnutfactors[j];
 
-GRbestvariance = GRtotalquanta * GRtotalquanta * (nutcount + 2);
-for (i = 0; i < foodcount; i++) GRfoodslots[i] = 0;
-solve_it(GRtotalquanta,GRtotalquanta,0,foodcount,nutcount);
+for (i = 0; i < GRfoodcount; i++)
+ {
+ if (parentheses[i] > 1)
+  {
+  for (j = i + 1; j < GRfoodcount; j++) if (parentheses[j] < 0) break;
+  while (parentheses[i] > 1 && parentheses[j] < -1)
+   {
+   parentheses[i]--;
+   parentheses[j]++;
+   }
+  }
+ }
 
-for (i = 0; i < foodcount; i++) foodlist[i]->grams = GRsaveslots[i] * food_work.grams / GRtotalquanta;
+for (i = GRfoodcount - 1; i >= 0; i--)
+ {
+ if (parentheses[i] < 0)
+  {
+  cumlevel -= parentheses[i];
+  GRfoodlevel[i] = cumlevel;
+  }
+ else if (parentheses[i] == 0)
+  {
+  GRfoodlevel[i] = cumlevel;
+  }
+ else if (parentheses[i] > 0)
+  {
+  GRfoodlevel[i] = cumlevel;
+  cumlevel -= parentheses[i];
+  }
+ }
+
+for (i = GRfoodcount - 1; i >= 0; i--)
+ {
+ if (parentheses[i] < 0)
+  {
+  for (j = GRfoodlevel[i] + parentheses[i]; j < GRfoodlevel[i]; j++) remaininggroupsassigner[j]++;
+  for (j = GRfoodlevel[i]; j < DV_COUNT; j++) remaininggroupsassigner[j] = 1;
+  }
+ else remaininggroupsassigner[GRfoodlevel[i]]++;
+ for (j = GRfoodlevel[i]; j >= 0; j--) GRgroupsremaining[i][j] = remaininggroupsassigner[j];
+ }
+
+GRlowestlevelchange[0] = 0;
+for (i = GRfoodcount - 1; i > 0; i--)
+ {
+ for (j = 0; j <= GRfoodlevel[i]; j++)
+  {
+  if (GRgroupsremaining[i-1][j] == GRgroupsremaining[i][j]) continue;
+  GRlowestlevelchange[i] = j;
+  break;
+  }
+ }
+
+GRbestvariance = GRtotalquanta * GRtotalquanta * (GRnutcount + 2);
+
+for (i = 0; i < GRfoodcount; i++) GRfoodslots[i] = 0;
+if (GRfoodcount > 10) GRtotalquanta = 40;
+levelcumbuf[0] = levelminbuf[0] = GRtotalquanta;
+solve_it(0,levelcumbuf,levelminbuf);
+
+for (i = 0; i < GRfoodcount; i++) foodlist[i]->grams = GRsaveslots[i] * food_work.grams / GRtotalquanta;
 
 header("NUT:  Add a Labeled Food");
 savegramoption = options.grams;
 options.grams = 1;
-recipe_show(1);
+recipe_show(1,1);
 options.grams = savegramoption;
 options.screen = 0;
 for (i = 0; i < NUTRIENT_COUNT; i++) recipe_root.nutrient[i] = NoData;
@@ -555,21 +688,59 @@ token = NULL;
 while (recipe_root.next != NULL) modify_recipe_food(1,token);
 }
 
-void solve_it(int prevquantum, int quanta, int foodno, int foodcount, int nutcount)
+void solve_it(int foodno, int *oldlevelcumbuf, int *oldlevelminbuf)
 {
 float newvariance;
-int max, min, x, i, j;
-max = quanta < prevquantum ? quanta : prevquantum;
-min = (quanta / (foodcount - foodno)) + ((quanta % (foodcount - foodno)) > 0 ? 1 : 0);
+int max, min, x, i, j, prevquanta, availquanta;
+int levelcumbuf[DV_COUNT], levelminbuf[DV_COUNT], progressfactor = 0;
+
+for (i = 0; i <= GRfoodlevel[foodno]; i++) levelcumbuf[i] = oldlevelcumbuf[i];
+for (i = 0; i <= GRfoodlevel[foodno]; i++) levelminbuf[i] = oldlevelminbuf[i];
+
+prevquanta = levelcumbuf[GRlowestlevelchange[foodno]];
+for (i = GRlowestlevelchange[foodno]; i <= GRfoodlevel[foodno]; i++) levelcumbuf[i] = 0;
+availquanta = levelminbuf[0] - levelcumbuf[0];
+
+max = availquanta < prevquanta ? availquanta : prevquanta;
+
+if (GRlowestlevelchange[foodno] <= 1)
+ {
+ levelminbuf[1] = (availquanta / GRgroupsremaining[foodno][1]) + ((availquanta % GRgroupsremaining[foodno][1]) > 0 ? 1 : 0);
+ for (i = 2; i <= GRfoodlevel[foodno]; i++) levelminbuf[i] = (levelminbuf[i-1] / GRgroupsremaining[foodno][i]) + ((levelminbuf[i-1] % GRgroupsremaining[foodno][i]) > 0 ? 1 : 0);
+ }
+
+else
+ {
+ levelminbuf[GRlowestlevelchange[foodno]] = ((levelminbuf[GRlowestlevelchange[foodno]-1] - levelcumbuf[GRlowestlevelchange[foodno]-1]) / GRgroupsremaining[foodno][GRlowestlevelchange[foodno]]) + (((levelminbuf[GRlowestlevelchange[foodno]-1] - levelcumbuf[GRlowestlevelchange[foodno]-1]) % GRgroupsremaining[foodno][GRlowestlevelchange[foodno]]) > 0 ? 1 : 0);
+ for (i = GRlowestlevelchange[foodno] + 1; i <= GRfoodlevel[foodno]; i++) levelminbuf[i] = (levelminbuf[i-1] / GRgroupsremaining[foodno][i]) + ((levelminbuf[i-1] % GRgroupsremaining[foodno][i]) > 0 ? 1 : 0);
+ }
+
+min = levelminbuf[GRfoodlevel[foodno]];
+if (min < 0) min = 0;
+
+if (!foodno)
+ {
+ progressfactor = min + 79;
+ printf("\nStarting calculation...\n");
+ }
+
+for (i = 0; i <= GRfoodlevel[foodno]; i++) levelcumbuf[i] += max + 1;
+
 for (x = max; x >= min; x--)
  {
  newvariance = 0;
+ for (i = 0; i <= GRfoodlevel[foodno]; i++) levelcumbuf[i]--;
+ if (!foodno)
+  {
+  for (i = 0; i < progressfactor - x; i++) printf("=");
+  if (i > 0) printf(">\n");
+  }
  GRfoodslots[foodno] = x;
- for (i = foodno + 1; i < foodcount; i++) GRfoodslots[i] = 0;
- if (quanta - x > 0) solve_it(x, quanta - x, foodno + 1, foodcount, nutcount);
+ for (i = foodno + 1; i < GRfoodcount; i++) GRfoodslots[i] = 0;
+ if (availquanta - x > 0) solve_it(foodno + 1, levelcumbuf, levelminbuf);
  else
   {
-  for (j = 0; j < nutcount; j++)
+  for (j = 0; j < GRnutcount; j++)
    {
    GRnutslots[j] = 0 - GRtotalquanta;
    for (i = 0; i <= foodno; i++) GRnutslots[j] += GRfoodslots[i] * GRquantamatrix[j][i];
@@ -578,13 +749,12 @@ for (x = max; x >= min; x--)
   if (newvariance < GRbestvariance)
    {
    GRbestvariance = newvariance;
-   for (i = 0; i < foodcount; i++) GRsaveslots[i] = GRfoodslots[i];
+   for (i = 0; i < GRfoodcount; i++) GRsaveslots[i] = GRfoodslots[i];
    }
   }
  }
 }
 
-
 void modify_servings()
 {
 char buf[1000];
@@ -650,7 +820,7 @@ for ( ; ; )
    spacer(3);
    while (junk == 'n' || junk == 'N')
     {
-    printf("\nType new weight of serving {xxxg for grams, xxxo for oz}:  ");
+    printf("\nType new weight of serving {#g for grams, #o for oz}:  ");
     get_recipe_qty(&food_work.grams);
     header("NUT:  Modify Serving Sizes");
     printf("%-60s   Grams  %8.2f\n",food_ptr->name,food_work.grams);
