@@ -2,7 +2,7 @@
 
 /*
     NUT nutrition software 
-    Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 by Jim Jozwiak.
+    Copyright (C) 1996-2007 by Jim Jozwiak.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -230,7 +230,9 @@ while (max > 0)
       DVMap[count+1] == FAMS || DVMap[count+1] == FAPU || 
       DVMap[count+1] == CHOLE || DVMap[count+1] == OMEGA6 ||
       DVMap[count+1] == CHO_NONFIB || DVMap[count+1] == OMEGA3 ||
-      DVMap[count+1] == CHOCDF || 
+      DVMap[count+1] == CHOCDF || DVMap[count+1] == VITD ||
+      DVMap[count+1] == PANTAC || DVMap[count+1] == FE   ||
+      DVMap[count+1] == CU     ||
       food_work.nutrient[DVMap[count+1]] / DV[DVMap[count+1]] >= .9999) deficpct[count] = 0;
    else 
     {
@@ -282,21 +284,16 @@ while (max > 0)
       serving_ratio = deficpct[count] / thispct;
       if (serving_ratio < serving_ratio_abacus[abacuscount] && serving_ratio * food_ptr->grams > 14) serving_ratio_abacus[abacuscount] = serving_ratio;
       }
-     if (strstr(food_ptr->name,"HYDR") != NULL) serving_ratio_abacus[abacuscount] = 0;
-     if (strstr(food_ptr->name,"LEAVENING AGENTS") != NULL) serving_ratio_abacus[abacuscount] = 0;
-     if (strstr(food_ptr->name,"RENNIN,TABLETS") != NULL) serving_ratio_abacus[abacuscount] = 0;
      if (strstr(food_ptr->name,"DEHYD") != NULL) high_grams = 114;
      if (strstr(food_ptr->name,"DRY") != NULL) high_grams = 114;
      if (strstr(food_ptr->name,"FROZEN DSSRT") != NULL) high_grams = 114;
-     if (strstr(food_ptr->name,"CEREALS RTE") != NULL) serving_ratio_abacus[abacuscount] = 0;
      if (strstr(food_ptr->name,"ALASKA NATIVE") != NULL) serving_ratio_abacus[abacuscount] = 0;
-     if (strstr(food_ptr->name,"CEREALS") != NULL && strstr(food_ptr->name,"RTE") != NULL) serving_ratio_abacus[abacuscount] = 0;
-     if (strstr(food_ptr->name,"INFANT FORMULA") != NULL) serving_ratio_abacus[abacuscount] = 0;
-     if (strstr(food_ptr->name,"INF FORMULA") != NULL) serving_ratio_abacus[abacuscount] = 0;
-     if (strstr(food_ptr->name,"CHILD FORMULA") != NULL) serving_ratio_abacus[abacuscount] = 0;
-     if (strstr(food_ptr->name,"ENSURE PLUS") != NULL) serving_ratio_abacus[abacuscount] = 0;
-     if (strstr(food_ptr->name,"BABYFOOD") != NULL) serving_ratio_abacus[abacuscount] = 0;
+     if (strstr(food_ptr->name,"AK NATIVE") != NULL) serving_ratio_abacus[abacuscount] = 0;
+     if (strstr(food_ptr->name,"INST BRKFST PDR") != NULL) serving_ratio_abacus[abacuscount] = 0;
+     if (strstr(food_ptr->name,"ENSURE") != NULL) serving_ratio_abacus[abacuscount] = 0;
+     if (strstr(food_ptr->name,"MEATLESS") != NULL) serving_ratio_abacus[abacuscount] = 0;
      if (strstr(food_ptr->name,"MARGARINE") != NULL) high_grams = 14;
+     if (food_ptr->fdgrp == 2 || food_ptr->fdgrp == 3 || food_ptr->fdgrp == 8 || food_ptr->fdgrp == 14 || food_ptr->fdgrp == 19 || food_ptr->fdgrp == 25) serving_ratio_abacus[abacuscount] = 0;
      if (serving_ratio_abacus[abacuscount] > 1 && high_grams == 114) serving_ratio_abacus[abacuscount] = 1;
      if ((serving_ratio_abacus[abacuscount] * food_ptr->grams) > high_grams) serving_ratio_abacus[abacuscount] = high_grams / food_ptr->grams;
      for (count = 0; count <= deficnuts; count++)
