@@ -2,7 +2,7 @@
 
 /*
     NUT nutrition software 
-    Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 by Jim Jozwiak.
+    Copyright (C) 1996-2007 by Jim Jozwiak.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "options.h"
 #include "nutrient.h"
 #include "abbrev.h"
+#include "fdgrp.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -49,40 +50,45 @@ LEU_G,LYS_G,MET_G,PHE_G,PRO_G,SER_G,THR_G,TRP_G,TYR_G,VAL_G
 
 int MiscMap[] =
 {
-27,ENERC_KJ,ASH,WATER,CAFFN,THEBRN,ALC,FOLAC,FOLFD,FOLDFE,RETOL,VITA_RAE,
+30,ENERC_KJ,ASH,WATER,CAFFN,THEBRN,ALC,
+FLD,BETN,CHOLN,FOLAC,FOLFD,FOLDFE,RETOL,VITA_RAE,
 VITB12_ADDED,VITE_ADDED,
 TOCPHA,TOCPHB,TOCPHG,TOCPHD,CARTA,CARTB,CRYPX,LUT_ZEA,LYCPN,
 CHOLE,PHYSTR,SITSTR,CAMD5,STID7
 };
 
-int SatMap[] =
+int SatMonoMap[] =
 {
-15,FASAT,F4D0,F6D0,F8D0,F10D0,F12D0,F13D0,F14D0,F15D0,F16D0,
-F17D0,F18D0,F20D0,F22D0,F24D0
+27,FASAT,F4D0,F6D0,F8D0,F10D0,F12D0,F13D0,F14D0,F15D0,F16D0,
+F17D0,F18D0,F20D0,F22D0,F24D0,
+FAMS,F14D1,F15D1,F16D1,F16D1C,F17D1,
+F18D1,F18D1C,F20D1,F22D1,F22D1C,F24D1C
 };
 
-int UnsatMap[] =
+int PolyTransMap[] =
 {
-40,FAMS,F14D1,F15D1,F16D1,F16D1C,F17D1,F18D1,F18D1C,F20D1,F22D1,F22D1C,F24D1C,
-FAPU,F18D2,F18D2CN6,F18D3,F18D3CN3,F18D3CN6,F18D4,F20D2CN6,F20D3,F20D3N6,
+29,FAPU,F18D2,F18D2CN6,F18D3,F18D3CN3,F18D3CN6,F18D4,F20D2CN6,F20D3,
+F20D3N3,F20D3N6,
 F20D4,F20D4N6,F20D5,F21D5,F22D4,F22D5,F22D6,
 FATRN,FATRNM,F16D1T,F18D1T,F22D1T,FATRNP,F18D2T,F18D2TT,F18D2I,F18D2CLA,F18D3I
 };
 
 int *ScreenMap[] =
 {
-DVMap,DVMap,CarbAminoMap,MiscMap,SatMap,UnsatMap
+DVMap,DVMap,CarbAminoMap,MiscMap,SatMonoMap,PolyTransMap
 };
 
 char *ScreenTitle[] =
 {
 "Daily Value Percentages","Daily Value Absolute Amounts",
 "Carbohydrates and Amino Acids","Miscellaneous Nutrients",
-"Saturated Fatty Acids","Unsaturated Fatty Acids"
+"Saturated and Monounsaturated Fatty Acids","Polyunsaturated and Trans Fatty Acids"
 };
 
 int MaxScreen = 6;
 
+int MaxFdGrp = FD_GRP_COUNT;
+
 float DVBase[NUTRIENT_COUNT];
 
 float DV[NUTRIENT_COUNT];
@@ -131,12 +137,12 @@ DVBase[VITC] = 60;
 DVBase[FASAT] = 20;
 DVBase[CHOLE] = 300;
 DVBase[VITD] = 400;
-DVBase[LA] = 4.44;
-DVBase[AA] = 0.2;
+DVBase[LA] = 12;
+DVBase[AA] = 0.4;
 DVBase[OMEGA6] = DVBase[LA] + DVBase[AA];
-DVBase[ALA] = 2.22;
-DVBase[EPA] = 0.33;
-DVBase[DHA] = 0.33;
+DVBase[ALA] = 1.555;
+DVBase[EPA] = 0.4;
+DVBase[DHA] = 0.4;
 DVBase[OMEGA3] = DVBase[ALA] + DVBase[EPA] + DVBase[DHA];
 DVBase[FAPU] = DVBase[OMEGA6] + DVBase[OMEGA3];
 DVBase[FAMS] = 61.5 - DVBase[FASAT] - DVBase[FAPU];
@@ -377,7 +383,6 @@ else
     strncpy(buffer,bufptr,nlength);
     buffer[nlength] = '\0';
     }
-   else;
    }
   if (tokenunique == 1)
    {
@@ -474,7 +479,6 @@ else
    else if (i+(7*s)>l) printf(col[7],i,fsptr[i],i+s,fsptr[i+s],i+(2*s),fsptr[i+(2*s)],i+(3*s),fsptr[i+(3*s)],i+(4*s),fsptr[i+(4*s)],i+(5*s),fsptr[i+(5*s)],i+(6*s),fsptr[i+(6*s)]);
    else if (i+(8*s)>l) printf(col[8],i,fsptr[i],i+s,fsptr[i+s],i+(2*s),fsptr[i+(2*s)],i+(3*s),fsptr[i+(3*s)],i+(4*s),fsptr[i+(4*s)],i+(5*s),fsptr[i+(5*s)],i+(6*s),fsptr[i+(6*s)],i+(7*s),fsptr[i+(7*s)]);
    else if (i+(9*s)>l) printf(col[9],i,fsptr[i],i+s,fsptr[i+s],i+(2*s),fsptr[i+(2*s)],i+(3*s),fsptr[i+(3*s)],i+(4*s),fsptr[i+(4*s)],i+(5*s),fsptr[i+(5*s)],i+(6*s),fsptr[i+(6*s)],i+(7*s),fsptr[i+(7*s)],i+(8*s),fsptr[i+(8*s)]);
-   else;
    }
   spacer(savelinecount < 19 ? savelinecount : 19);
   printf("Type number of food category (\"b\" to go back, <enter> to quit):  ");
@@ -649,7 +653,7 @@ switch (options.screen)
   p6 = 902 * food_work.nutrient[SHORT6] / food_work.nutrient[ENERC_KCAL];
   h3 = 902 * food_work.nutrient[LONG3] / food_work.nutrient[ENERC_KCAL];
   h6 = 902 * food_work.nutrient[LONG6] / food_work.nutrient[ENERC_KCAL];
-  o  = 902 * (food_work.nutrient[FASAT] + food_work.nutrient[FAMS]) / food_work.nutrient[ENERC_KCAL];
+  o  = 902 * food_work.nutrient[FAT] * options.fattyacidfactor / food_work.nutrient[ENERC_KCAL] - food_work.nutrient[SHORT6] - food_work.nutrient[LONG6] - food_work.nutrient[SHORT3] - food_work.nutrient[LONG3];
   fprintf(fp,"Omega-6 HUFA     %5.1f%%\n",n6hufa(p3,p6,h3,h6,o));
   }
  break;
@@ -674,46 +678,39 @@ switch (options.screen)
  break;
  case 3 :
  fprintf(fp,"Miscellaneous Nutrients\n\n");
- for (i = 1; i < 6 ; i++)
+ for (i = 1; i < 11 ; i++)
   {
   food_display_line(fp,ScreenMap[options.screen][i],0,0,0,0);
-  food_display_line(fp,ScreenMap[options.screen][i+13],0,0,0,0);
-  food_display_line(fp,ScreenMap[options.screen][i+22],0,1,0,0);
+  food_display_line(fp,ScreenMap[options.screen][i+10],0,0,0,0);
+  food_display_line(fp,ScreenMap[options.screen][i+20],0,1,0,0);
   }
- for (i = 6; i < 10 ; i++)
-  {
-  food_display_line(fp,ScreenMap[options.screen][i],0,0,0,0);
-  food_display_line(fp,ScreenMap[options.screen][i+13],0,1,0,0);
-  }
- for (i = 10; i <= 13 ; i++) food_display_line(fp,ScreenMap[options.screen][i],0,1,0,0);
+ fprintf(fp,"\n\n\n");
  break;
  case 4 :
- fprintf(fp,"Saturated Fatty Acids\n\n");
- for (i = 1; i < 8 ; i++)
-  {
-  food_display_line(fp,ScreenMap[options.screen][i],0,0,0,0);
-  food_display_line(fp,ScreenMap[options.screen][i+8],0,1,0,0);
-  }
- food_display_line(fp,ScreenMap[options.screen][i],0,1,0,0);
- fprintf(fp,"\n\n\n\n\n");
- break;
- case 5 :
- fprintf(fp,"%-27s","Unsaturated Fatty Acids");
- food_display_line(fp,ScreenMap[options.screen][13],0,0,1,0);
- food_display_line(fp,ScreenMap[options.screen][28],0,1,1,0);
- fprintf(fp,"%-27s"," ");
- food_display_line(fp,ScreenMap[options.screen][14],0,0,1,0);
- food_display_line(fp,ScreenMap[options.screen][29],0,1,1,0);
  food_display_line(fp,ScreenMap[options.screen][1],0,0,1,0);
- food_display_line(fp,ScreenMap[options.screen][15],0,1,1,0);
- for (i = 2; i < 13 ; i++)
+ fprintf(fp,"Saturated and Monounsaturated Fatty Acids\n");
+ food_display_line(fp,ScreenMap[options.screen][2],0,1,1,0);
+ for (i = 3; i < 15 ; i++)
   {
   food_display_line(fp,ScreenMap[options.screen][i],0,0,1,0);
-  food_display_line(fp,ScreenMap[options.screen][i+14],0,0,1,0);
-  food_display_line(fp,ScreenMap[options.screen][i+28],0,1,1,0);
+  food_display_line(fp,ScreenMap[options.screen][i+13],0,1,1,0);
   }
- fprintf(fp,"%-27s"," ");
- food_display_line(fp,ScreenMap[options.screen][27],0,1,1,0);
+ food_display_line(fp,ScreenMap[options.screen][i],0,1,1,0);
+ break;
+ case 5 :
+ fprintf(fp,"%-27s","Polyunsaturated and Trans Fatty Acids\n\n");
+ for (i = 1; i < 10 ; i++)
+  {
+  food_display_line(fp,ScreenMap[options.screen][i],0,0,1,0);
+  food_display_line(fp,ScreenMap[options.screen][i+9],0,0,1,0);
+  food_display_line(fp,ScreenMap[options.screen][i+18],0,1,1,0);
+  }
+ for (i = 10; i < 12 ; i++)
+  {
+  fprintf(fp,"%-54s"," ");
+  food_display_line(fp,ScreenMap[options.screen][i+18],0,1,1,0);
+  }
+ fprintf(fp,"\n\n");
  break;
  default :
  if (! test_for_negative_zero(&food_work.nutrient[ScreenMap[options.screen][1]])) fprintf(fp,"Calories(%4.0f)   %5.0f%%     ",DV[ScreenMap[options.screen][1]],food_work.nutrient[ScreenMap[options.screen][1]]/DV[ScreenMap[options.screen][1]]*100);
@@ -770,7 +767,7 @@ switch (options.screen)
   p6 = 902 * food_work.nutrient[SHORT6] / food_work.nutrient[ENERC_KCAL];
   h3 = 902 * food_work.nutrient[LONG3] / food_work.nutrient[ENERC_KCAL];
   h6 = 902 * food_work.nutrient[LONG6] / food_work.nutrient[ENERC_KCAL];
-  o  = 902 * (food_work.nutrient[FASAT] + food_work.nutrient[FAMS]) / food_work.nutrient[ENERC_KCAL];
+  o  = 902 * food_work.nutrient[FAT] * options.fattyacidfactor / food_work.nutrient[ENERC_KCAL] - food_work.nutrient[SHORT6] - food_work.nutrient[LONG6] - food_work.nutrient[SHORT3] - food_work.nutrient[LONG3];
   fprintf(fp,"Omega-6 HUFA     %5.0f%%\n",n6hufa(p3,p6,h3,h6,o));
   }
  break;
@@ -808,7 +805,7 @@ switch (flags)
   if (! test_for_negative_zero(&food_work.nutrient[nut])) fprintf(fp,"%8.1f %-3s\n",food_work.nutrient[nut],Unit[nut]);
   else fprintf(fp,"%9s%-3s\n","(nd)"," ");
 #else
-  if (nut != VITE && nut != VITK && nut != ILE_G && nut != CARTB && nut != CRYPX && nut != LUT_ZEA && nut != LYCPN && nut != VITA_RAE && nut != RETOL && nut != VITA_RAE && nut != VITB12_ADDED && nut != VITE_ADDED && nut != F14D0)
+  if (nut != VITE && nut != VITK && nut != ILE_G)
    {
    if (! test_for_negative_zero(&food_work.nutrient[nut])) fprintf(fp,"%7.1f %-3s\n",food_work.nutrient[nut],Unit[nut]);
    else fprintf(fp,"%8s%-3s\n","(nd)"," ");
@@ -825,7 +822,7 @@ switch (flags)
   if (! test_for_negative_zero(&food_work.nutrient[nut])) fprintf(fp,"%8.2f %-3s\n",food_work.nutrient[nut],Unit[nut]);
   else fprintf(fp,"%9s%-3s\n","(nd)"," ");
 #else
-  if (nut != F18D2CN6 && nut != F20D2CN6 && nut != F20D3 && nut != F20D4 && nut != F20D5 && nut != F22D4)
+  if (nut != F4D0 && nut != F18D2CN6 && nut != F20D2CN6 && nut != F20D3 && nut != F20D4 && nut != F20D5 && nut != F22D4 && nut != F24D0)
    {
    if (! test_for_negative_zero(&food_work.nutrient[nut])) fprintf(fp,"%7.2f %-3s\n",food_work.nutrient[nut],Unit[nut]);
    else fprintf(fp,"%8s%-3s\n","(nd)"," ");
@@ -880,6 +877,57 @@ while (recipe_ptr->next != NULL)
  }
 }
 
+void modify_label_food(int num, char *command)
+{
+struct food *m = NULL, *pm = NULL, *am = NULL, *po = NULL, *ao = NULL, *recipe_ptr = &recipe_root;
+int count = 0;
+int action;
+action = evaluate_action(command);
+if (num == action) return;
+while (recipe_ptr->next != NULL)
+ {
+ count++;
+ if (count == action && action > 0 && num > action)
+  {
+  ao = recipe_ptr;
+  po = recipe_ptr->next;
+  }
+ if (count == action && action > 0 && num < action)
+  {
+  ao = recipe_ptr->next;
+  if (recipe_ptr->next != NULL) po = recipe_ptr->next->next;
+  else po = NULL;
+  }
+ if (count == num)
+  {
+  m = recipe_ptr->next;
+  pm = recipe_ptr->next->next;
+  am = recipe_ptr;
+  if (action == 0)
+   {
+   recipe_ptr->next = recipe_ptr->next->next;
+   free(m);
+   return;
+   }
+  if (action < 0)
+   {
+   if (action == -1 && m->refuse >= 0) m->refuse++;
+   if (action == -1 && m->refuse <  0) m->refuse = 1;
+   if (action == -2 && m->refuse <= 0) m->refuse--;
+   if (action == -2 && m->refuse >  0) m->refuse = -1;
+   if (action == -3 && m->refuse <  0) m->refuse++;
+   if (action == -3 && m->refuse >  0) m->refuse--;
+   return;
+   }
+  }
+ recipe_ptr = recipe_ptr->next;
+ }
+if (num > count || action > count) return;
+am->next = pm;
+ao->next = m;
+m->next = po;
+}
+
 void compute_derived_fields(struct food *food_ptr)
 {
 float ratio;
@@ -895,7 +943,9 @@ if (food_ptr->nutrient[ALA] < 0) food_ptr->nutrient[ALA] = 0;
 food_ptr->nutrient[SHORT3] = food_ptr->nutrient[ALA];
 food_ptr->nutrient[SHORT3] += food_ptr->nutrient[F18D4];
 food_ptr->nutrient[LONG6] = food_ptr->nutrient[F20D2CN6];
-food_ptr->nutrient[LONG6] += food_ptr->nutrient[F20D3];
+if (food_ptr->nutrient[F20D3N3] > 0) food_ptr->nutrient[LONG3] += food_ptr->nutrient[F20D3N3];
+if (food_ptr->nutrient[F20D3N6] > 0) food_ptr->nutrient[LONG6] += food_ptr->nutrient[F20D3N6];
+if (food_ptr->nutrient[F20D3N3] == 0 && food_ptr->nutrient[F20D3N6] == 0) food_ptr->nutrient[LONG6] += food_ptr->nutrient[F20D3];
 food_ptr->nutrient[AA] = food_ptr->nutrient[F20D4];
 if (food_ptr->nutrient[F20D4N6] > 0) food_ptr->nutrient[AA] = food_ptr->nutrient[F20D4N6];
 food_ptr->nutrient[LONG6] += food_ptr->nutrient[AA];
@@ -907,7 +957,7 @@ food_ptr->nutrient[LONG3] += food_ptr->nutrient[F22D6];
 if (food_ptr->nutrient[SHORT3] == 0 && test_for_negative_zero(&food_ptr->nutrient[ALA]) && test_for_negative_zero(&food_ptr->nutrient[F18D4])) food_ptr->nutrient[SHORT3] = NoData;
 if (food_ptr->nutrient[LONG3] == 0 && test_for_negative_zero(&food_ptr->nutrient[EPA]) && test_for_negative_zero(&food_ptr->nutrient[F22D5]) && test_for_negative_zero(&food_ptr->nutrient[DHA])) food_ptr->nutrient[LONG3] = NoData;
 if (food_ptr->nutrient[SHORT6] == 0 && test_for_negative_zero(&food_ptr->nutrient[LA]) && test_for_negative_zero(&food_ptr->nutrient[F18D3CN6])) food_ptr->nutrient[SHORT6] = NoData;
-if (food_ptr->nutrient[LONG6] == 0 && test_for_negative_zero(&food_ptr->nutrient[EPA]) && test_for_negative_zero(&food_ptr->nutrient[F22D5]) && test_for_negative_zero(&food_ptr->nutrient[DHA])) food_ptr->nutrient[LONG6] = NoData;
+if (food_ptr->nutrient[LONG6] == 0 && test_for_negative_zero(&food_ptr->nutrient[F20D2CN6]) && test_for_negative_zero(&food_ptr->nutrient[F20D3]) && test_for_negative_zero(&food_ptr->nutrient[F20D3N6]) && test_for_negative_zero(&food_ptr->nutrient[AA])) food_ptr->nutrient[LONG6] = NoData;
 food_ptr->nutrient[OMEGA6] = food_ptr->nutrient[SHORT6] + food_ptr->nutrient[LONG6];
 if (food_ptr->nutrient[OMEGA6] == 0 && test_for_negative_zero(&food_ptr->nutrient[SHORT6]) && test_for_negative_zero(&food_ptr->nutrient[LONG6])) food_ptr->nutrient[OMEGA6] = NoData;
 food_ptr->nutrient[OMEGA3] = food_ptr->nutrient[SHORT3] + food_ptr->nutrient[LONG3];
