@@ -2,7 +2,7 @@
 
 /*
     NUT nutrition software 
-    Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 by Jim Jozwiak.
+    Copyright (C) 1996-2007 by Jim Jozwiak.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include <string.h>
 
 int Minut = -1;
+int Fdgrp = -1;
 
 void view_nuts(int opt)
 {
@@ -61,6 +62,23 @@ for ( ; ; )
   strcat(head,Minut == -1 ? "?" : Nutrient[Minut]);
   header(head);
   }
+ if (opt == 6)
+  {
+  strcpy(head,"NUT:  Rank Foods by Nutrients within ");
+  strcat(head,Fdgrp == -1 ? "?" : FdGrp[Fdgrp]);
+  header(head);
+  fdgrp_list();
+  if (Fdgrp == -1)
+   {
+   printf("\nType number of Food Group (or <enter> to quit):  ");
+   num = get_int();
+   if (num < 1 || num > MaxFdGrp) return;
+   Fdgrp = FdGrpMap[num-1];
+   }
+  strcpy(head,"NUT:  Rank Foods by Nutrients within ");
+  strcat(head,Fdgrp == -1 ? "?" : FdGrp[Fdgrp]);
+  header(head);
+  }
  nut_list();
  printf("\nType number of Nutrient to see food rankings (or <enter> to quit):  ");
  num = get_int();
@@ -104,6 +122,8 @@ if (opt == 3 && options.screen > 0) strcat(head," per 100 Grams Dry Weight");
 if (opt == 4 && options.screen > 0) strcat(head," per Serving");
 if (opt == 2 && options.screen > 0) strcat(head," per Daily Recorded Meals");
 if (opt == 5) sprintf(head,"%-49s%14s %14s","NUT:  Foods Ranked by Nutrients per Serving:",Nutrient[num],Nutrient[Minut]);
+if (opt == 6 && options.screen > 0) sprintf(head,"%-21s %1s %2s %-1s","NUT:  Foods Ranked by",Nutrient[num],"in 100g",FdGrp[Fdgrp]);
+if (opt == 6 && options.screen == 0) sprintf(head,"%-21s %1s %2s %-1s","NUT:  Foods Ranked by %DV",Nutrient[num],"in 100g",FdGrp[Fdgrp]);
 if (opt == 2 && meal_count(options.temp_meal_root) == 0)
  {
  header(head); 
@@ -151,6 +171,13 @@ if (opt == 1) for (count = 0 ; count < FoodCount ; count++)
  if (options.screen == 0) abacus[count] = 100 * food_ptr->nutrient[num] / DV[num];
  else abacus[count] = food_ptr->nutrient[num];
  }
+if (opt == 6) for (count = 0 ; count < FoodCount ; count++)
+ {
+ food_ptr = food_ptr->next;
+ if (options.screen == 0) abacus[count] = 100 * food_ptr->nutrient[num] / DV[num];
+ else abacus[count] = food_ptr->nutrient[num];
+ if (food_ptr->fdgrp != Fdgrp) abacus[count] = -1;
+ }
 if (opt == 5)
  {
  for (count = 0 ; count < FoodCount ; count++) 
@@ -254,40 +281,43 @@ while (displayed <= (FoodCount - 20) && (junk == 'm' || junk == 'M'))
    if (options.grams) printf("%-54s        %7.1f g %5.0f%%\n",thisname,theseounces*28.35,abacus[max]);
    if (!options.grams) printf("%-54s        %6.1f oz %5.0f%%\n",thisname,theseounces,abacus[max]);
    }
-  if (opt == 2 && options.screen > 0 && options.screen !=5)
+  if (opt == 2 && options.screen > 0 && options.screen < 4)
    {
    theseounces = abacus[max] / (food_ptr->nutrient[num] * 28.35);
    if (options.grams) printf("%-54s %7.1f g %9.1f %-3s\n",thisname,theseounces*28.35,abacus[max]/100,Unit[num]);
    if (!options.grams) printf("%-54s %6.1f oz %9.1f %-3s\n",thisname,theseounces,abacus[max]/100,Unit[num]);
    }
-  if (opt == 2 && options.screen > 0 && options.screen ==5)
+  if (opt == 2 && options.screen > 3)
    {
    theseounces = abacus[max] / (food_ptr->nutrient[num] * 28.35);
    if (options.grams) printf("%-54s %7.1f g %9.2f %-3s\n",thisname,theseounces*28.35,abacus[max]/100,Unit[num]);
    if (!options.grams) printf("%-54s %6.1f oz %9.2f %-3s\n",thisname,theseounces,abacus[max]/100,Unit[num]);
    }
   if (opt == 1 && options.screen == 0) printf("%-54s                  %5.0f%%\n",thisname,abacus[max]);
-  if (opt == 1 && options.screen > 0 && options.screen !=4) printf("%-54s          %10.1f %-3s\n",thisname,abacus[max],Unit[num]);
-  if (opt == 1 && options.screen > 0 && options.screen ==4) printf("%-54s          %10.2f %-3s\n",thisname,abacus[max],Unit[num]);
+  if (opt == 1 && options.screen > 0 && options.screen < 4) printf("%-54s          %10.1f %-3s\n",thisname,abacus[max],Unit[num]);
+  if (opt == 1 && options.screen > 0 && options.screen > 3) printf("%-54s          %10.2f %-3s\n",thisname,abacus[max],Unit[num]);
+  if (opt == 6 && options.screen == 0) printf("%-54s                  %5.0f%%\n",thisname,abacus[max]);
+  if (opt == 6 && options.screen > 0 && options.screen < 4) printf("%-54s          %10.1f %-3s\n",thisname,abacus[max],Unit[num]);
+  if (opt == 6 && options.screen > 0 && options.screen > 3) printf("%-54s          %10.2f %-3s\n",thisname,abacus[max],Unit[num]);
   if (opt == 3 && options.screen == 0 && options.grams) printf("%-54s        %7.1f g %5.0f%%\n",thisname,10000 / (100 - food_ptr->nutrient[WATER]),abacus[max]);
   if (opt == 3 && options.screen == 0 && !options.grams) printf("%-54s        %6.1f oz %5.0f%%\n",thisname,10000 / (100 - food_ptr->nutrient[WATER]) / 28.35,abacus[max]);
-  if (opt == 3 && options.screen > 0 && options.screen !=4 && options.grams) printf("%-54s %7.1f g %9.1f %-3s\n",thisname,10000 / (100 - food_ptr->nutrient[WATER]),abacus[max],Unit[num]);
-  if (opt == 3 && options.screen > 0 && options.screen !=4 && !options.grams) printf("%-54s %6.1f oz %9.1f %-3s\n",thisname,10000 / (100 - food_ptr->nutrient[WATER]) / 28.35,abacus[max],Unit[num]);
-  if (opt == 3 && options.screen > 0 && options.screen ==4 && options.grams) printf("%-54s %7.1f g %9.2f %-3s\n",thisname,10000 / (100 - food_ptr->nutrient[WATER]),abacus[max],Unit[num]);
-  if (opt == 3 && options.screen > 0 && options.screen ==4 && !options.grams) printf("%-54s %6.1f oz %9.2f %-3s\n",thisname,10000 / (100 - food_ptr->nutrient[WATER]) / 28.35,abacus[max],Unit[num]);
+  if (opt == 3 && options.screen > 0 && options.screen < 4 && options.grams) printf("%-54s %7.1f g %9.1f %-3s\n",thisname,10000 / (100 - food_ptr->nutrient[WATER]),abacus[max],Unit[num]);
+  if (opt == 3 && options.screen > 0 && options.screen < 4 && !options.grams) printf("%-54s %6.1f oz %9.1f %-3s\n",thisname,10000 / (100 - food_ptr->nutrient[WATER]) / 28.35,abacus[max],Unit[num]);
+  if (opt == 3 && options.screen > 0 && options.screen > 3 && options.grams) printf("%-54s %7.1f g %9.2f %-3s\n",thisname,10000 / (100 - food_ptr->nutrient[WATER]),abacus[max],Unit[num]);
+  if (opt == 3 && options.screen > 0 && options.screen > 3 && !options.grams) printf("%-54s %6.1f oz %9.2f %-3s\n",thisname,10000 / (100 - food_ptr->nutrient[WATER]) / 28.35,abacus[max],Unit[num]);
   if (opt == 0 && options.screen == 0 && options.grams) printf("%-54s        %7.1f g %5.0f%%\n",thisname,10000 / food_ptr->nutrient[ENERC_KCAL],abacus[max]);
   if (opt == 0 && options.screen == 0 && !options.grams) printf("%-54s        %6.1f oz %5.0f%%\n",thisname,10000 / food_ptr->nutrient[ENERC_KCAL] / 28.35,abacus[max]);
-  if (opt == 0 && options.screen > 0 && options.screen !=4 && options.grams) printf("%-54s %7.1f g %9.1f %-3s\n",thisname,10000 / food_ptr->nutrient[ENERC_KCAL],abacus[max],Unit[num]);
-  if (opt == 0 && options.screen > 0 && options.screen !=4 && !options.grams) printf("%-54s %6.1f oz %9.1f %-3s\n",thisname,10000 / food_ptr->nutrient[ENERC_KCAL] / 28.35,abacus[max],Unit[num]);
-  if (opt == 0 && options.screen > 0 && options.screen ==4 && options.grams) printf("%-54s %7.1f g %9.2f %-3s\n",thisname,10000 / food_ptr->nutrient[ENERC_KCAL],abacus[max],Unit[num]);
-  if (opt == 0 && options.screen > 0 && options.screen ==4 && !options.grams) printf("%-54s %6.1f oz %9.2f %-3s\n",thisname,10000 / food_ptr->nutrient[ENERC_KCAL] / 28.35,abacus[max],Unit[num]);
+  if (opt == 0 && options.screen > 0 && options.screen < 4 && options.grams) printf("%-54s %7.1f g %9.1f %-3s\n",thisname,10000 / food_ptr->nutrient[ENERC_KCAL],abacus[max],Unit[num]);
+  if (opt == 0 && options.screen > 0 && options.screen < 4 && !options.grams) printf("%-54s %6.1f oz %9.1f %-3s\n",thisname,10000 / food_ptr->nutrient[ENERC_KCAL] / 28.35,abacus[max],Unit[num]);
+  if (opt == 0 && options.screen > 0 && options.screen > 3 && options.grams) printf("%-54s %7.1f g %9.2f %-3s\n",thisname,10000 / food_ptr->nutrient[ENERC_KCAL],abacus[max],Unit[num]);
+  if (opt == 0 && options.screen > 0 && options.screen > 3 && !options.grams) printf("%-54s %6.1f oz %9.2f %-3s\n",thisname,10000 / food_ptr->nutrient[ENERC_KCAL] / 28.35,abacus[max],Unit[num]);
   if (opt == 5) printf("%-52s %7.1f %-3s    %7.1f %-3s\n",thisname,food_ptr->grams / 100 * food_ptr->nutrient[num],Unit[num],food_ptr->grams / 100 * food_ptr->nutrient[Minut],Unit[Minut]);
   if (opt == 4 && options.screen == 0 && options.grams) printf("%-54s        %7.1f g %5.0f%%\n",thisname,food_ptr->grams,abacus[max]);
   if (opt == 4 && options.screen == 0 && !options.grams) printf("%-54s        %6.1f oz %5.0f%%\n",thisname,food_ptr->grams / 28.35,abacus[max]);
-  if (opt == 4 && options.screen > 0 && options.screen !=4 && options.grams) printf("%-54s %7.1f g %9.1f %-3s\n",thisname,food_ptr->grams,abacus[max],Unit[num]);
-  if (opt == 4 && options.screen > 0 && options.screen !=4 && !options.grams) printf("%-54s %6.1f oz %9.1f %-3s\n",thisname,food_ptr->grams / 28.35,abacus[max],Unit[num]);
-  if (opt == 4 && options.screen > 0 && options.screen ==4 && options.grams) printf("%-54s %7.1f g %9.2f %-3s\n",thisname,food_ptr->grams,abacus[max],Unit[num]);
-  if (opt == 4 && options.screen > 0 && options.screen ==4 && !options.grams) printf("%-54s %6.1f oz %9.2f %-3s\n",thisname,food_ptr->grams / 28.35,abacus[max],Unit[num]);
+  if (opt == 4 && options.screen > 0 && options.screen < 4 && options.grams) printf("%-54s %7.1f g %9.1f %-3s\n",thisname,food_ptr->grams,abacus[max],Unit[num]);
+  if (opt == 4 && options.screen > 0 && options.screen < 4 && !options.grams) printf("%-54s %6.1f oz %9.1f %-3s\n",thisname,food_ptr->grams / 28.35,abacus[max],Unit[num]);
+  if (opt == 4 && options.screen > 0 && options.screen > 3 && options.grams) printf("%-54s %7.1f g %9.2f %-3s\n",thisname,food_ptr->grams,abacus[max],Unit[num]);
+  if (opt == 4 && options.screen > 0 && options.screen > 3 && !options.grams) printf("%-54s %6.1f oz %9.2f %-3s\n",thisname,food_ptr->grams / 28.35,abacus[max],Unit[num]);
   abacus[max] = 0;
   max = max_array(abacus);
   }
@@ -335,3 +365,17 @@ for ( count = 0 ; count <= *ScreenMap[options.screen] / 2 - (*ScreenMap[options.
  }
 spacer(count);
 }
+
+void fdgrp_list()
+{
+int count;
+int intdiv = MaxFdGrp / 2 + (MaxFdGrp % 2 == 0 ? 0 : 1);
+printf("\n\n");
+for ( count = 0 ; count <= MaxFdGrp / 2 - (MaxFdGrp % 2 == 0 ? 1 : 0); count++)
+ {
+ printf("%2d. %-33s %1s ",count+1,FdGrp[FdGrpMap[count]]," ");
+ if (count+intdiv < MaxFdGrp) printf("%2d. %-33s\n",count+1+intdiv,FdGrp[FdGrpMap[count+intdiv]]);
+ if (count+intdiv >= MaxFdGrp) printf("\n");
+ }
+spacer(count+2);
+}
