@@ -2,7 +2,7 @@
 
 /*
     NUT nutrition software 
-    Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006 by Jim Jozwiak.
+    Copyright (C) 1996-2007 by Jim Jozwiak.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -52,6 +52,7 @@ for ( ; ; )
  spacer(8);
  printf("\nType meal date (yyyymmdd) or just press <enter> for today's date:  ");
  get_string(new_meal_date,8);
+ if (new_meal_date[0] == '-' || new_meal_date[0] == '+') time_machine(new_meal_date);
  if (strlen(new_meal_date) == 8) strcpy(meal_date,new_meal_date);
  if (strlen(new_meal_date) == 8 || strlen(new_meal_date) == 0) break;
  }
@@ -95,7 +96,7 @@ for ( ; ; )
   else
    {
    modfood = atoi(key);
-   strncpy(buff,key,128);
+   strncpy(buff,key,60);
    token = strtok(buff, ", ");
    token = strtok(NULL,", ");
    if (strcmp(key,"") == 0 || modfood == 0) break;
@@ -188,3 +189,32 @@ meal_date[8] = '\0';
 if (meal_date[6] == ' ') meal_date[6] = '0';
 strncpy(whatever,meal_date,9);
 }
+
+void time_machine(char *whatever)
+{
+int c, month = 0, time_travel;
+struct tm *p;
+time_t t;
+char timestamp[26];
+char meal_date[9];
+char *thismonth;
+char *Months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
+                "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
+time_travel = 86400 * atoi(whatever);
+time(&t);
+t += time_travel;
+p = localtime(&t);
+strcpy(timestamp,asctime(p));
+meal_date[0] = timestamp[20];
+meal_date[1] = timestamp[21];
+meal_date[2] = timestamp[22];
+meal_date[3] = timestamp[23];
+thismonth = timestamp + 4;
+for (c = 0; c < 12 ; c++) if (strncmp(thismonth,Months[c],3) == 0) month = c+1;
+sprintf(meal_date+4,"%02d",month);
+meal_date[6] = timestamp[8];
+meal_date[7] = timestamp[9];
+meal_date[8] = '\0';
+if (meal_date[6] == ' ') meal_date[6] = '0';
+strncpy(whatever,meal_date,9);
+}
