@@ -25,7 +25,6 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <math.h>
             
 /* replacement compatibility C99-style math.h functions
    tested with gcc on: i386, amd64, ia64, powerpc, sparc, s390, hppa, alpha
@@ -103,7 +102,7 @@ char buff[128];
 float lastresult = *result;
 float dividend, divisor;
 if ( lastresult <= 0 ) lastresult = 1;
-printf("\n# servings, xxxg grams, xxxo oz, xxxc calories, \"b\" back:  ");
+printf("\n# servings, #g grams, #o oz, #c calories, \"b\" back:  ");
 fgets(buff,128,stdin);
 if (strchr(buff,'g') != NULL) 
  {
@@ -154,6 +153,21 @@ else return atof(qtystring) * food_ptr->grams;
 write_OPTIONS();
 }
 
+int evaluate_action(char *actionstring)
+{
+int newpos;
+if (strchr(actionstring,'(') != NULL) return -1;
+else if (strchr(actionstring,')') != NULL) return -2;
+else if (strchr(actionstring,'!') != NULL) return -3;
+else if (strchr(actionstring,'m') != NULL)
+ {
+ newpos = atoi(strchr(actionstring,'m')+1);
+ if (newpos < 1) newpos = 1;
+ return newpos;
+ }
+else return 0;
+}
+
 void get_select(float *result)
 {
 char buff[128];
@@ -196,7 +210,7 @@ int count;
 size_t size = 80;
 size -= strlen(string);
 size /= 2;
-for (count = 0 ; count < size ; count++) buffer[count] = ' '; 
+for (count = 0 ; (size_t) count < size ; count++) buffer[count] = ' '; 
 buffer[count] = '\0';
 #ifndef DOS
 if (subuser[0] != '\0') printf("\033[2J%s%s\n[DB] %s\n",buffer,string,subuser);
